#include "Error.h"

#include <QDebug>

#include "ErrorDisplayer.h"

Error::Error(ErrorDisplayer* _displayer, QString _title, QString _description,
             ErrorType _type, int msecDuration)
    : QWidget()
{

    displayer = _displayer;
    type      = _type;

    layout.setSpacing(0);
    layout.setMargin(0);
    // setMaximumHeight(100);
    setMaximumWidth(250);

    QString style =
        "QWidget {background-color: rgba(0,0,0,0); color: black; margin: 0px; "
        "padding: 0px;} QLabel {margin: 0px; padding: 10px; padding-bottom: "
        "10px; border-radius: 0px} ";
    if (type == ErrorType::ET_ERROR)
    {
        style.append("Error {background-color: rgba(221, 74, 55, 0.75);}");
        titlePrefix.setText("Error");
    }
    else if (type == ErrorType::ET_WARNING)
    {
        style.append("Error {background-color: rgba(221, 204, 55, 0.75);}");
        titlePrefix.setText("Warning");
    }
    else if (type == ErrorType::ET_INFO)
    {
        style.append("Error {background-color: rgba(55, 207, 221, 0.75);}");
        titlePrefix.setText("Info");
    }
    this->setStyleSheet(style);

    titlePrefix.setStyleSheet("QLabel {font-weight: bold; padding-right: 0px}");
    layout.addWidget(&titlePrefix, 0, 0);

    title.setText(_title);
    title.setStyleSheet("QLabel {font-weight: bold; padding:0px}");
    layout.addWidget(&title, 0, 1);

    layout.addWidget(&countLabel, 0, 2, Qt::AlignRight);

    description.setText(_description);
    description.setStyleSheet("QLabel {padding-top:0px}");
    description.setWordWrap(true);
    layout.addWidget(&description, 1, 0, 1, 3);

    count = 0;
    incrementCount(msecDuration);

    setLayout(&layout);
    adjustSize();
}

Error::~Error() { displayer->removeError(this); }

void Error::incrementCount(int msecDuration)
{
    count++;
    if (count > 1)
    {
        countLabel.setText(QString::number(count));
    }
    QTimer* timer = new QTimer(this);
    connect(timer, &QTimer::timeout, this, &Error::decrementCount);
    timer->start(msecDuration);
}

void Error::decrementCount()
{
    count -= 1;
    countLabel.setText(QString::number(count));
    if (!count)
    {
        delete this;
    }
}

bool Error::compareError(QString title, QString description, ErrorType type)
{
    return !QString::compare(this->title.text(), title) &&
           !QString::compare(this->description.text(), description) &&
           this->type == type;
}

void Error::mouseReleaseEvent(QMouseEvent* e)
{
    if (e->button() & Qt::LeftButton)
    {
        this->deleteLater();
    }
}

void Error::paintEvent(QPaintEvent* event)
{
    QStyleOption opt;
    opt.init(this);
    QPainter p(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
    QWidget::paintEvent(event);
}
