#include "ErrorDisplayer.h"

#include <QDebug>

ErrorDisplayer::ErrorDisplayer(QWidget* _parent) : QWidget(_parent)
{
    parent = _parent;
    index  = 0;

    layout.setDirection(QBoxLayout::BottomToTop);
    setLayout(&layout);
    layout.setSizeConstraint(QLayout::SetFixedSize);
    reposition();
}

ErrorDisplayer::~ErrorDisplayer()
{
    while (!errors.isEmpty())
    {
        delete errors.takeFirst();
    }
}

void ErrorDisplayer::createError(QString title, QString description,
                                 ErrorType type, int msecDuration)
{
    if (!errors.isEmpty())
    {
        Error* lastError = errors.last();
        if (lastError->compareError(title, description, type))
        {
            lastError->incrementCount(msecDuration);
            return;
        }
    }
    Error* error = new Error(this, title, description, type, msecDuration);
    layout.addWidget(error);
    errors.append(error);
    error->show();
    reposition();
}

void ErrorDisplayer::removeError(Error* error)
{
    layout.removeWidget(error);
    errors.removeAll(error);
    reposition();
}

void ErrorDisplayer::reposition()
{
    adjustSize();
    move(parent->width() - width(), parent->height() - height());
}
