#pragma once

#include <QAbstractButton>
#include <QtWidgets>

class ToggleButton : public QAbstractButton
{
    Q_OBJECT
    Q_PROPERTY(int offset READ offset WRITE setOffset)
    Q_PROPERTY(QBrush brush READ brush WRITE setBrush)
    Q_PROPERTY(bool state READ state WRITE setState)

public:
    ToggleButton(QWidget* parent = nullptr);
    explicit ToggleButton(const QBrush& brush, QWidget* parent = nullptr);

    QSize sizeHint() const override;

    QBrush brush() const { return _brush; }

    void setBrush(const QBrush& brsh) { _brush = brsh; }

    int offset() const { return _x; }
    void setOffset(int o)
    {
        _x = o;
        update();
    }

    bool state() { return _switch; }
    void setState(bool s) { changeState(s); }

signals:
    void toggled(bool state);

protected:
    void paintEvent(QPaintEvent*) override;
    void mouseReleaseEvent(QMouseEvent*) override;
    void enterEvent(QEvent*) override;
    void changeState(bool state);

private:
    bool _switch;
    qreal _opacity;
    int _x, _y, _height, _margin;
    QBrush _thumb, _track, _brush;
    QPropertyAnimation* _anim = nullptr;
};
