#include "Field.h"

#include <QRegExp>

Field::Field() : type(Type::EMPTY) {}

Field::Field(int64_t value) : type(Type::INT), signedInteger(value) {}

Field::Field(uint64_t value) : type(Type::UINT), unsignedInteger(value) {}

Field::Field(double value) : type(Type::DOUBLE), floatingPoint(value) {}

Field::Field(QString value) : type(Type::STRING), string(value) {}

bool Field::operator==(const Field& toCompare) const
{
    if (type != toCompare.type)
        return false;

    switch (type)
    {
        case Type::INT:
            return signedInteger == toCompare.signedInteger;
        case Type::UINT:
            return unsignedInteger == toCompare.unsignedInteger;
        case Type::DOUBLE:
            return floatingPoint == toCompare.floatingPoint;
        case Type::STRING:
            return string == toCompare.string;
        case Type::EMPTY:
            return true;
    }

    return false;
}

Field::Type Field::getType() const { return type; }

int64_t Field::getInteger() const
{
    if (type == Type::INT)
        return signedInteger;
    else if (type == Type::UINT)
        return unsignedInteger;
    else if (type == Type::DOUBLE)
        return floatingPoint;
    else
        return 0;
}

uint64_t Field::getUnsignedInteger() const
{
    if (type == Type::UINT)
        return unsignedInteger;
    else if (type == Type::INT)
        return signedInteger;
    else if (type == Type::DOUBLE)
        return floatingPoint;
    else
        return 0;
}

double Field::getDouble() const
{
    if (type == Type::DOUBLE)
        return floatingPoint;
    else if (type == Type::INT)
        return signedInteger;
    else if (type == Type::UINT)
        return unsignedInteger;
    else
        return 0;
}

QString Field::toString() const
{
    switch (type)
    {
        case Type::INT:
            return QString::number(signedInteger);
        case Type::UINT:
            return QString::number(unsignedInteger);
        case Type::DOUBLE:
        {
            QString str = QString::number(floatingPoint, 'f', 8);

            // Remove any number of trailing 0's
            str.remove(QRegExp("0+$"));

            // If the last character is just a '.' then remove it
            str.remove(QRegExp("\\.$"));

            return str;
        }
        case Type::STRING:
            return string;
        default:
            return QString();
    }
};