#include "MessageBroker.h"

#include <Core/Module/Module.h>

void MessageBroker::subscribe(Filter filter, Module* observer,
                              Callback callback)
{
    observers.insert(filter, {observer, new Callback(std::move(callback))});

    // Register the beforeDelete slot of the module to automatically remove the
    // subscriber
    connect(observer->getEventHandler(), &EventHandler::beforeDelete, this,
            &MessageBroker::onModuleDeleted);
}

void MessageBroker::unsubscribe(Filter filter, Module* module)
{
    auto callbackList = observers.values(filter);

    for (auto& elem : callbackList)
    {
        if (elem.first == module)
            observers.remove(filter, elem);
    }
}

void MessageBroker::publish(const Message& message)
{
    for (auto filter : observers.keys())
    {
        // If the message matches the filed, then notify all its subscribers
        if (filter.match(message))
        {
            for (auto subscriber : observers.values(filter))
            {
                auto copy = message;
                filter.filterMessage(copy);
                (*subscriber.second)(copy, filter);
            }
        }
    }
}

void MessageBroker::onModuleDeleted(Module* module)
{
    for (auto filter : observers.keys())
    {
        for (auto subscriber : observers.values(filter))
        {
            if (subscriber.first == module)
            {
                observers.remove(filter, subscriber);
                delete subscriber.second;
            }
        }
    }
}
