#pragma once

#include <Core/Message/Filter.h>

#include <QObject>
#include <functional>

class Module;

class MessageBroker : public QObject
{
    Q_OBJECT

public:
    using Callback = std::function<void(const Message&, const Filter&)>;

    void subscribe(Filter filter, Module* observer, Callback callback);

    /**
     * @brief Unsubscribe the module from the filter.
     */
    void unsubscribe(Filter filter, Module* module);

    /**
     * @brief Publish a message to all its subscribers.
     */
    void publish(const Message& message);

private:
    QMultiMap<Filter, QPair<Module*, Callback*>> observers;

private slots:
    void onModuleDeleted(Module* module);
};
