#include "CompactCommandPad.h"

CompactCommandPad::CompactCommandPad(QWidget* parent)
    : DefaultModule(parent), timer(nullptr), continuosSendTimeout(false)
{
    setupUi();
    defaultContextMenuSetup();
}

CompactCommandPad::~CompactCommandPad() { delete selector; }

QWidget* CompactCommandPad::toWidget() { return this; }

XmlObject CompactCommandPad::toXmlObject()
{
    XmlObject obj = XmlObject("compact_command_pad");
    selector->toXmlObject(obj);
    return obj;
}

void CompactCommandPad::fromXmlObject(const XmlObject& obj)
{
    if (obj.getObjectName() == "compact_command_pad")
        selector->fromXmlObject(obj);

    QString label;
    selected =
        selector->getSelection(label, selectedMessage, continuosSendTimeout);

    if (selected)
    {
        if (continuosSendTimeout != 0)
            button->setText(label + " [Start]");

        button->setText(label);
    }
}

void CompactCommandPad::buttonEditActionTriggered()
{
    selector->exec();
    selected =
        selector->getSelection(label, selectedMessage, continuosSendTimeout);

    if (timer)
    {
        timer->stop();
        timer->deleteLater();
        timer = nullptr;
    }

    if (selected)
    {
        if (continuosSendTimeout != 0)
            button->setText(label + " [Start]");
        else
            button->setText(label);
    }
}

void CompactCommandPad::setupUi()
{
    QVBoxLayout* outerLayout = new QVBoxLayout;
    outerLayout->setContentsMargins(2, 2, 2, 2);

    button = new QPushButton("Edit me");
    outerLayout->addWidget(button);
    connect(button, &QPushButton::clicked,
            [=]()
            {
                if (timer)
                {
                    timer->stop();
                    timer->deleteLater();
                    timer = nullptr;
                    button->setText(label + " [Start]");
                }
                else
                {
                    if (selected)
                    {
                        if (continuosSendTimeout == 0)
                        {
                            send();
                        }
                        else
                        {
                            button->setText(label + " [Stop]");

                            send();
                            timer = new QTimer(this);
                            connect(timer, &QTimer::timeout, this,
                                    &CompactCommandPad::send);
                            timer->start(continuosSendTimeout);
                        }
                    }
                }
            });

    setLayout(outerLayout);

    selector = new CommandSelector(this);
}

void CompactCommandPad::send()
{
    getCore()->getMessageBroker()->publish(selectedMessage);
}

void CompactCommandPad::addCustomActionsToMenu()
{
    QAction* edit = new QAction("Edit button", this);
    addActionToMenu(edit);
    connect(edit, &QAction::triggered, this,
            &CompactCommandPad::buttonEditActionTriggered);
}
