#include "CsvLogger.h"

#include <Components/FilterSelector/FilterSelector.h>

CsvLogger::CsvLogger(QWidget* parent) : DefaultModule(parent), started(false), file(nullptr) {
    defaultContextMenuSetup();
    setupUi();
}

CsvLogger::~CsvLogger() { }

QWidget* CsvLogger::toWidget(){
    return this;
}

XmlObject CsvLogger::toXmlObject(){
    XmlObject obj(getName(ModuleId::CSV_LOGGER));
    return obj;
}

void CsvLogger::fromXmlObject(const XmlObject& obj){
    Q_UNUSED(obj);
}

void CsvLogger::setupUi(){
    QVBoxLayout* layout = new QVBoxLayout;

    QComboBox* combo = new QComboBox;
    combo->addItems(FilterSelector::listOfMessages().keys());
    layout->addWidget(combo);

    QHBoxLayout* bottomLayout = new QHBoxLayout;

    QLineEdit* filenameEdit = new QLineEdit;
    filenameEdit->setPlaceholderText("Type output file name here.");
    bottomLayout->addWidget(filenameEdit);

    QPushButton* startStopLogging = new QPushButton("Start logging");
    bottomLayout->addWidget(startStopLogging);

    QObject::connect(startStopLogging, &QPushButton::clicked, [this, combo, filenameEdit, startStopLogging](){
        if(!started){
            if(file){
                file->close();
                file->deleteLater();
            }

            file = new QFile(filenameEdit->text());
            if(!file->open(QIODevice::WriteOnly | QIODevice::Text | QIODevice::Append)){
                file->deleteLater();
                file = nullptr;

                error("I/O Error", QString("Unable to open file \"") + filenameEdit->text() + "\".");
                return;
            }

            file->write("pc_timestamp,");
            const auto& properties = FilterSelector::listOfMessages()[combo->currentText()];
            for(int i = 0; i < properties.size(); i++){
                file->write((properties[i] + (i < properties.size()-1 ? "," : "\n")).toUtf8());
            }

            getCore()->getMessageBroker()->unsubscribe(lastSub, this);
            lastSub = Filter::fromString(combo->currentText());
            getCore()->getMessageBroker()->subscribe(
                lastSub, this,
                [this](const Message& message, const Filter& filter)
                { received(message); });
            started = true;
        } else {
            if(file){
                file->close();
                file->deleteLater();
                file = nullptr;
            }
            started = false;
            getCore()->getMessageBroker()->unsubscribe(lastSub, this);
        }

        startStopLogging->setText(started ? "Stop logging" : "Start logging");
    });

    layout->addLayout(bottomLayout);
    setLayout(layout);
}

void CsvLogger::received(const Message& msg){
    if(!file)
        return;

    auto timestamp = QDateTime::currentSecsSinceEpoch();
    file->write((QString::number(timestamp) + ",").toUtf8());

    const auto& properties = FilterSelector::listOfMessages()[lastSub.getTopic().toString()];
    for(int i = 0; i < properties.size(); i++){
        file->write((msg.getField(properties[i]).toString() + (i < properties.size()-1 ? "," : "\n")).toUtf8());
    }
}
