#include "DefaultModule.h"

#include <Components/ContextMenuSeparator/ContextMenuSeparator.h>
#include <Core/ModulesManager/ModulesManager.h>

#include <QCloseEvent>

DefaultModule::DefaultModule(QWidget* parent) : Module(parent) {}

DefaultModule::~DefaultModule()
{
    for (int i = 0; i < menuActions.count(); i++)
    {
        delete menuActions[i];
    }
    menuActions.clear();
}

void DefaultModule::defaultContextMenuSetup()
{
    if (areDefaultSettingsDone)
        return;

    areDefaultSettingsDone = true;
    setContextMenuPolicy(Qt::CustomContextMenu);
    connect(this, &DefaultModule::customContextMenuRequested, this,
            &DefaultModule::onCustomContextMenuRequested);
    addDefaultActionsToMenu();
    addCustomActionsToMenu();
}

void DefaultModule::addCustomActionsToMenu() {}

void DefaultModule::onSkywardHubContextMenuRequested(QMenu& menu,
                                                     const QPoint& pos)
{
    QMenu newMenu(menuName);
    if (!menuName.isEmpty())
    {
        newMenu.addActions(menuActions);
        menu.addMenu(&newMenu);
    }
    else
        menu.addActions(menuActions);

    menu.addSeparator();

    emit getEventHandler()->contextMenuRequest(menu, pos);
}

SkywardHubCoreProxy& DefaultModule::getCore() { return proxyCore; }

void DefaultModule::onCustomContextMenuRequested(const QPoint& pos)
{
    QMenu menu;
    onSkywardHubContextMenuRequested(menu, mapToGlobal(pos));
}

QString DefaultModule::getName(const ModuleId id)
{
    return getCore()->getModulesManager()->getModuleName(id);
}

Window* DefaultModule::getWindow()
{
    QObject* parent = this;
    while (dynamic_cast<Window*>(parent) == NULL)
    {
        parent = parent->parent();
    }
    return dynamic_cast<Window*>(parent);
}

void DefaultModule::error(QString title, QString description, int msecDuration)
{
    getWindow()->errorDisplayer->createError(title, description,
                                             ErrorType::ET_ERROR, msecDuration);
}

void DefaultModule::warning(QString title, QString description,
                            int msecDuration)
{
    getWindow()->errorDisplayer->createError(
        title, description, ErrorType::ET_WARNING, msecDuration);
}

void DefaultModule::info(QString title, QString description, int msecDuration)
{
    getWindow()->errorDisplayer->createError(title, description,
                                             ErrorType::ET_INFO, msecDuration);
}

void DefaultModule::closeEvent(QCloseEvent* event)
{
    if (event->spontaneous())
    {
        emit getEventHandler()->beforeDelete(this);
        QWidget::closeEvent(event);
    }
    else
    {
        QWidget::closeEvent(event);
    }
}

void DefaultModule::addDefaultActionsToMenu()
{
    QAction* close = new QAction("Close");
    connect(close, &QAction::triggered, this, &DefaultModule::onCloseClicked);

    QAction* replace = new QAction("Replace");
    connect(replace, &QAction::triggered, this,
            &DefaultModule::onReplaceClicked);

    addActionToMenu(close);
    addActionToMenu(replace);
}

void DefaultModule::addActionToMenu(QAction* action)
{
    menuActions.append(action);
}

void DefaultModule::showMenuActionsInSeparatedMenu(const QString& menuName)
{
    this->menuName = menuName;
}

void DefaultModule::onReplaceClicked()
{
    Module* newModule =
        getCore()->getModulesManager()->invokeModulesPickerService();
    if (newModule != nullptr)
        emit getEventHandler()->replaceMeWith(this, newModule);
}

void DefaultModule::onCloseClicked() { close(); }
