#pragma once

#include <Components/ErrorDisplayer/ErrorDisplayer.h>
#include <Core/Module/Module.h>
#include <Core/SkywardHubCore.h>
#include <Modules/MainWindow/Window.h>
#include <Modules/ModuleInfo.h>

class DefaultModule : public Module
{
    Q_OBJECT

public:
    static constexpr int LONG_ERROR_DURATION   = 1500;
    static constexpr int MEDIUM_ERROR_DURATION = 10000;
    static constexpr int SHORT_ERROR_DURATION  = 5000;

    DefaultModule(QWidget* parent = nullptr);
    ~DefaultModule();

    virtual void defaultContextMenuSetup();
    virtual void addCustomActionsToMenu();
    virtual void onSkywardHubContextMenuRequested(QMenu& menu, const QPoint& p);

    SkywardHubCoreProxy& getCore();
    virtual void onCustomContextMenuRequested(const QPoint& p);

    QString getName(const ModuleId id);

    Window* getWindow();

    void error(QString title, QString description,
               int duration = MEDIUM_ERROR_DURATION);
    void warning(QString title, QString description,
                 int duration = MEDIUM_ERROR_DURATION);
    void info(QString title, QString description,
              int duration = MEDIUM_ERROR_DURATION);

protected:
    void closeEvent(QCloseEvent* event) override;

    void addDefaultActionsToMenu();
    void addActionToMenu(QAction* action);

    void showMenuActionsInSeparatedMenu(const QString& menuName);
    virtual void onReplaceClicked();
    virtual void onCloseClicked();

private:
    SkywardHubCoreProxy proxyCore;
    QList<QAction*> menuActions;

    QString menuName;
    bool areDefaultSettingsDone = false;
};
