#pragma once

#include <Core/Message/Filter.h>
#include <Core/QCustomPlot/QCustomPlot.h>
#include <Modules/DefaultModule/DefaultModule.h>

#include <QList>
#include <QTimer>
#include <QWidget>

class Graph : public DefaultModule
{
    Q_OBJECT

    static constexpr int MAX_DATA_AGE = 30 * 60;  // [s]

public:
    explicit Graph(QWidget* parent = nullptr);

    QWidget* toWidget() override;
    XmlObject toXmlObject() override;
    void fromXmlObject(const XmlObject& xmlObject) override;

private slots:
    void onSubscribeClicked();
    void onClearClicked();
    void onStopClicked(bool checked);
    void onFollowClicked(bool checked);

    void onFilterAdded(const Filter& filter);
    void onFilterRemoved(const Filter& filter);

    void onUpdateTimerTick();

private:
    void setupUi();
    void customContextMenuSetup();
    void onCustomContextMenuRequested(const QPoint& pos) override;

    QCustomPlot* plot;

    struct Line
    {
        QCPGraph* graph;
        QVector<double> bufferX;
        QVector<double> bufferY;
    };

    QMap<Filter, QPair<QMap<QString, Line>, bool>> lines;

    QTimer updaterTimer;
    int updatePeriod = 1000 / 5;  // 5fps

    bool stopped   = false;
    bool following = true;
};
