#include "MavlinkCommandAdapter.h"

#include <Core/ModulesManager/ModulesManager.h>

#include "Core/XmlObject.h"
#include "MavlinkModule.h"
#include "MavlinkVersionHeader.h"
#include "Modules/SkywardHubStrings.h"

MavlinkCommandAdapter::MavlinkCommandAdapter() {}

void MavlinkCommandAdapter::setSerialPort(QSerialPort *value)
{
    serial = value;
}

bool MavlinkCommandAdapter::encodeCommand(const Message &msg,
                                          mavlink_message_t &output)
{
    QString messageName = msg.getTopic().toString().replace(
        SkywardHubStrings::commandsTopic + "/", "");
    if (messageName == "PING_TC")
    {
        mavlink_msg_ping_tc_pack(
            MAV_SYS, MAV_CMP, &output,
            msg.getField("timestamp").getUnsignedInteger());
        return true;
    }
    else if (messageName == "COMMAND_TC")
    {
        mavlink_msg_command_tc_pack(
            MAV_SYS, MAV_CMP, &output,
            msg.getField("command_id").getUnsignedInteger());
        return true;
    }
    else if (messageName == "SYSTEM_TM_REQUEST_TC")
    {
        mavlink_msg_system_tm_request_tc_pack(
            MAV_SYS, MAV_CMP, &output,
            msg.getField("tm_id").getUnsignedInteger());
        return true;
    }
    else if (messageName == "SENSOR_TM_REQUEST_TC")
    {
        mavlink_msg_sensor_tm_request_tc_pack(
            MAV_SYS, MAV_CMP, &output,
            msg.getField("sensor_id").getUnsignedInteger());
        return true;
    }
    else if (messageName == "SERVO_TM_REQUEST_TC")
    {
        mavlink_msg_servo_tm_request_tc_pack(
            MAV_SYS, MAV_CMP, &output,
            msg.getField("servo_id").getUnsignedInteger());
        return true;
    }
    else if (messageName == "SET_SERVO_ANGLE_TC")
    {
        mavlink_msg_set_servo_angle_tc_pack(
            MAV_SYS, MAV_CMP, &output,
            msg.getField("servo_id").getUnsignedInteger(),
            msg.getField("angle").getDouble());
        return true;
    }
    else if (messageName == "WIGGLE_SERVO_TC")
    {
        mavlink_msg_wiggle_servo_tc_pack(
            MAV_SYS, MAV_CMP, &output,
            msg.getField("servo_id").getUnsignedInteger());
        return true;
    }
    else if (messageName == "RESET_SERVO_TC")
    {
        mavlink_msg_reset_servo_tc_pack(
            MAV_SYS, MAV_CMP, &output,
            msg.getField("servo_id").getUnsignedInteger());
        return true;
    }
    else if (messageName == "SET_REFERENCE_ALTITUDE_TC")
    {
        mavlink_msg_set_reference_altitude_tc_pack(
            MAV_SYS, MAV_CMP, &output,
            msg.getField("ref_altitude").getDouble());
        return true;
    }
    else if (messageName == "SET_REFERENCE_TEMPERATURE_TC")
    {
        mavlink_msg_set_reference_temperature_tc_pack(
            MAV_SYS, MAV_CMP, &output, msg.getField("ref_temp").getDouble());
        return true;
    }
    else if (messageName == "SET_DEPLOYMENT_ALTITUDE_TC")
    {
        mavlink_msg_set_deployment_altitude_tc_pack(
            MAV_SYS, MAV_CMP, &output,
            msg.getField("dpl_altitude").getDouble());
        return true;
    }
    else if (messageName == "SET_ORIENTATION_TC")
    {
        mavlink_msg_set_orientation_tc_pack(MAV_SYS, MAV_CMP, &output,
                                            msg.getField("yaw").getDouble(),
                                            msg.getField("pitch").getDouble(),
                                            msg.getField("roll").getDouble());
        return true;
    }
    else if (messageName == "SET_COORDINATES_TC")
    {
        mavlink_msg_set_coordinates_tc_pack(
            MAV_SYS, MAV_CMP, &output, msg.getField("latitude").getDouble(),
            msg.getField("longitude").getDouble());
        return true;
    }
    else if (messageName == "RAW_EVENT_TC")
    {
        mavlink_msg_raw_event_tc_pack(
            MAV_SYS, MAV_CMP, &output,
            msg.getField("topic_id").getUnsignedInteger(),
            msg.getField("event_id").getUnsignedInteger());
        return true;
    }
    else if (messageName == "SET_TARGET_COORDINATES_TC")
    {
        mavlink_msg_set_target_coordinates_tc_pack(
            MAV_SYS, MAV_CMP, &output, msg.getField("latitude").getDouble(),
            msg.getField("longitude").getDouble());
        return true;
    }
    else if (messageName == "SET_ALGORITHM_TC")
    {
        mavlink_msg_set_algorithm_tc_pack(
            MAV_SYS, MAV_CMP, &output,
            msg.getField("algorithm_number").getUnsignedInteger());
        return true;
    }
    return false;
}

void MavlinkCommandAdapter::send(mavlink_message_t msg)
{
    mavlinkWriter.setSerialPort(serial);
    mavlinkWriter.write(msg);
}

bool MavlinkCommandAdapter::produceMsgFromXml(const XmlObject &xml,
                                              mavlink_message_t *msg)
{
    bool result = false;
    if (xml.getObjectName() == "ACK_TM")
    {
        QString recv_string = xml.getAttribute("recv_msgid");
        QString seq_string  = xml.getAttribute("seq_ack");

        bool ok1, ok2;
        uint8_t recvId = recv_string.toUInt(&ok1);
        uint8_t seq    = seq_string.toUInt(&ok2);

        if (ok1 && ok2)
        {
            mavlink_msg_ack_tm_pack(MAV_SYS, MAV_CMP, msg, recvId, seq);
        }
        result = true;
    }

    return result;
}
