#pragma once

#include <Core/Module/Module.h>

#include <QSerialPort>
#include <QTimer>
#include <QWidget>

#include "Components/ToggleButton/ToggleButton.h"
#include "Core/Message/Message.h"
#include "MavlinkCommandAdapter.h"
#include "MavlinkReader.h"
#include "Modules/DefaultModule/DefaultModule.h"

Q_DECLARE_METATYPE(Message);

namespace Ui
{
class MavlinkModule;
}

class MavlinkModule : public DefaultModule
{
    Q_OBJECT

public:
    explicit MavlinkModule(QWidget *parent = nullptr);
    ~MavlinkModule();

    QWidget *toWidget() override;
    XmlObject toXmlObject() override;
    void fromXmlObject(const XmlObject &xmlObject) override;

    void onCommandReceived(const Message &msg);

public slots:
    void onStartClicked();
    void onStopClicked();
    void onMsgReceived(const Message &msg);
    void onLinkQualityTimerTick();

protected:
    mavlink_message_t parseMavlinkMsg(char *buffer, int readCount);

    void closePort();
    void updateLinkSignalIndicator();

    void onOpenLogFolderClick();

private slots:
    void onStartStreamToggled(bool state);

private:
    void setupUi();
    void initializeSerialPortView();
    bool startReadingOnSerialPort();

    ToggleButton startSerialStreamToggleButton;
    MavlinkReader mavlinkReader;
    MavlinkCommandAdapter mavlinkCommandAdapter;
    QSerialPort serialPort;

    QComboBox *portsComboBox;
    QComboBox *baudrateComboBox;
    QLabel *rateLabel;
    QCheckBox *logCheckBox;

    QTimer linkQualityTimer;
    const int linkQualityPeriod = 2000;  // [ms]
    int msgArrived              = 0;
    bool portOpen;
};
