#pragma once

#include <QDateTime>
#include <QFile>
#include <QSerialPort>

#include "Core/Message/Message.h"
#include "Core/XmlObject.h"
#include "MavlinkVersionHeader.h"

class MavlinkModule;

class MavlinkReader : public QObject
{
    Q_OBJECT
public:
    explicit MavlinkReader(MavlinkModule* parent);
    ~MavlinkReader();

    void startReading();
    void stopReading();

    Message generateMessage(const mavlink_message_t& msg);

    void setSerialPort(QSerialPort* port);
    void closeLog();
    void openLogFile();

    static const mavlink_message_info_t& getMessageFormat(uint8_t messageId);

signals:
    void msgReceived(const Message& msg);

private slots:
    void onReadyRead();

protected:
    /* convert a field of any type to a string */
    Field decodeField(const mavlink_message_t& msg,
                      const mavlink_field_info_t& field);

    /* if field is an array, decode every element in the array */
    Field decodeArrayElement(const mavlink_message_t& msg,
                             const mavlink_field_info_t& field, int idx);

    void setLogFilePath();

private:
    mavlink_message_t decodedMessage;
    mavlink_status_t mavlinkStatus;

    QSerialPort* serialPort = nullptr;
    MavlinkModule* module;

    QString logFilePath = "";
    QFile logFile;
};
