#include "MavlinkRocketMsgTestingModule.h"

#include <Core/MessageBroker/MessageBroker.h>

#include <QDebug>
#include <QMap>

#include "Core/XmlObject.h"
#include "MavlinkCommandAdapter.h"
#include "MavlinkReader.h"
#include "MavlinkVersionHeader.h"
#include "ui_MavlinkRocketMsgTestingModule.h"

MavlinkRocketMsgTestingModule::MavlinkRocketMsgTestingModule(QWidget* parent)
    : DefaultModule(parent), ui(new Ui::MavlinkRocketMsgTestingModule)
{
    ui->setupUi(this);
    defaultContextMenuSetup();
    init();
}

MavlinkRocketMsgTestingModule::~MavlinkRocketMsgTestingModule()
{
    delete ui;  // This delete all the radiobutton
}

void MavlinkRocketMsgTestingModule::init()
{
    ui->lineEdit_mavlinkInputFilePath->setText(defaultMavlinkFileName);
    connect(ui->pushButton_sendMsg, &QPushButton::clicked, this,
            &MavlinkRocketMsgTestingModule::onSendMsgClicked);
    connect(ui->pushButton_loadInputFile, &QPushButton::clicked, this,
            &MavlinkRocketMsgTestingModule::onLoadFileClicked);
}

QWidget* MavlinkRocketMsgTestingModule::toWidget() { return this; }

XmlObject MavlinkRocketMsgTestingModule::toXmlObject()
{
    return XmlObject(getName(ModuleId::MAVLINK_RCK_TESTING));
}

void MavlinkRocketMsgTestingModule::fromXmlObject(const XmlObject& xmlObject)
{
    Q_UNUSED(xmlObject);
}

void MavlinkRocketMsgTestingModule::createNewButton(const QString& txt,
                                                    QLayout* container)
{
    QRadioButton* radiobutton = new QRadioButton(txt);
    connectRadioButton(radiobutton);
    container->addWidget(radiobutton);
}

void MavlinkRocketMsgTestingModule::connectRadioButton(
    QRadioButton* radiobutton)
{
    connect(radiobutton, &QRadioButton::clicked, this,
            [this, radiobutton]() { this->onRadioButtonClicked(radiobutton); });
}

void MavlinkRocketMsgTestingModule::setSelectedMsg(const QString& msgName)
{
    if (currentMsgView != nullptr)
    {
        delete currentMsgView;
        currentMsgView = nullptr;
    }

    QList<XmlObject*> clickedMsg = mavlinkMsgDefinitionFile.deepSearchObjects(
        [this, msgName](const XmlObject* obj)
        {
            QString name = obj->getAttribute(xmlFieldName);
            if (name == msgName)
            {
                return true;
            }
            return false;
        });

    if (clickedMsg.count() == 1)
    {
        currentMsgView = new MsgView(clickedMsg[0], msgName);
        ui->details_layout->insertWidget(0, currentMsgView->getView());
    }
}

void MavlinkRocketMsgTestingModule::createViewFromXmlMessages(
    QList<XmlObject*> messagesList)
{
    for (int i = 0; i < messagesList.count(); i++)
    {
        XmlObject* xmlObj = messagesList[i];
        QString msgType   = xmlObj->getAttribute(xmlFieldType).trimmed();
        QGroupBox* gbox   = getOrCreateGBox(msgType);
        QString name      = xmlObj->getAttribute(xmlFieldName).trimmed();
        if (name != "")
        {
            createNewButton(xmlObj->getAttribute(xmlFieldName), gbox->layout());
        }
    }
}

QGroupBox* MavlinkRocketMsgTestingModule::getOrCreateGBox(const QString& title)
{
    QGroupBox* gBox = nullptr;
    if (title != "" && !viewGroupList.contains(title))
    {
        gBox                 = new QGroupBox(title);
        viewGroupList[title] = gBox;
        ui->msg_layout->insertWidget(0, gBox);
        gBox->setLayout(new QVBoxLayout());
    }
    else if (viewGroupList.contains(title))
    {
        gBox = viewGroupList[title];
    }
    return gBox;
}

QString MavlinkRocketMsgTestingModule::getCurrentTopic() const
{
    return ui->lineEdit_outputTopic->text();
}

void MavlinkRocketMsgTestingModule::onRadioButtonClicked(
    QRadioButton* radiobutton)
{
    setSelectedMsg(radiobutton->text());
}

void MavlinkRocketMsgTestingModule::clearView()
{
    if (currentMsgView != nullptr)
    {
        delete currentMsgView;
        currentMsgView = nullptr;
    }

    QList<QGroupBox*> groupBox = viewGroupList.values();
    for (int i = 0; i < groupBox.count(); i++)
        delete groupBox[i];

    viewGroupList.clear();
}

void MavlinkRocketMsgTestingModule::onLoadFileClicked()
{
    clearView();

    QString filePath = SkywardHubStrings::defaultConfigurationFolder + "/" +
                       ui->lineEdit_mavlinkInputFilePath->text().trimmed();
    if (mavlinkMsgDefinitionFile.loadFromFile(filePath))
    {
        QList<XmlObject*> xmlMessages =
            mavlinkMsgDefinitionFile.deepSearchObjects(
                [this](const XmlObject* msg)
                {
                    if (msg->getObjectName() == xmlChildName)
                    {
                        return true;
                    }
                    return false;
                });

        createViewFromXmlMessages(xmlMessages);
    }
}

XmlObject MavlinkRocketMsgTestingModule::msgToXml() const
{
    XmlObject xmlMsg;
    if (currentMsgView != nullptr)
    {
        xmlMsg.setObjectName(currentMsgView->getName());
        auto fields = currentMsgView->getFields();
        QMapIterator<QString, QLineEdit*> i(fields);
        while (i.hasNext())
        {
            i.next();
            xmlMsg.addAttribute(i.key(), i.value()->text());
        }
    }
    return xmlMsg;
}

void MavlinkRocketMsgTestingModule::onSendMsgClicked()
{
    if (currentMsgView != nullptr)
    {
        XmlObject xmlMsg = msgToXml();
        testMavlinkEncodeAndDecode(xmlMsg);
    }
}

bool MavlinkRocketMsgTestingModule::testMavlinkEncodeAndDecode(
    const XmlObject& xmlMsg)
{
    // Test mavlink msg encoding
    mavlink_message_t mavMsg;
    bool ok = false;
    ok      = MavlinkCommandAdapter().produceMsgFromXml(xmlMsg, &mavMsg);

    // Test of buffer writing
    //    if(MavlinkCommandAdapter().produceMsgFromXml(xmlMsg, &mavMsg)){
    //        unsigned char buff[sizeof(mavlink_message_t)+1];
    //        int msg_len = mavlink_msg_to_send_buffer(buff, &mavMsg);
    //        ok = msg_len > 0;
    //    }

    // Test mavlink msg decoding
    if (ok)
    {
        MavlinkReader mavlinkReader(nullptr);
        Message msgProduced = mavlinkReader.generateMessage(mavMsg);
        Topic currentTopic(getCurrentTopic().trimmed());
        getCore()->getMessageBroker()->publish(msgProduced);
    }
    return ok;
}

// mavlink_message_t MavlinkRocketMsgTestingModule::genericPack(string funcName,
// char* params[])
//{
//     mavlink_message_t result_msg;

//    if(!funcName.compare("mavlink_msg_sm_tm_pack")){
//        uint8_t system_id = (uint8_t)atoi(params[0]);
//        ecc..
//        mavlink_msg_sm_tm_pack(system_id, ecc...);
//    }
//    else if(!funcName.compare("mavlink_msg_gps_tm_pack")){
//        param...
//        mavlink_msg_gps_tm_pack(param...);
//    }
//    .
//    .
//    .

//    return result_msg;
//}

// _______________________________________________ MsgView
// ___________________________________________________

MsgView::MsgView(const XmlObject* obj, QString name) : QObject()
{
    this->name = name;
    bool ok;
    id = obj->getAttribute(xmlFieldId).trimmed().toInt(&ok);
    if (!ok)
    {
        id = 0;
    }

    for (int i = 0; i < obj->childCount(); i++)
    {
        XmlObject fieldXml = obj->childAt(i);
        if (fieldXml.getObjectName() == xmlFieldElementName)
        {
            //            auto i = fieldXml.attributesIterator();
            //            while(i.hasNext()){
            //                auto pair = i.next();
            //            }
            QString fieldName   = fieldXml.getAttribute(xmlFieldName);
            QLineEdit* lineEdit = new QLineEdit();
            fields[fieldName]   = lineEdit;
        }
        else if (fieldXml.getObjectName() == xmlFieldDescr)
        {
            description = fieldXml.getTextValue();
        }
    }
}

MsgView::~MsgView() { delete view; }

QWidget* MsgView::getView()
{
    if (view == nullptr)
    {
        view                = new QWidget();
        QVBoxLayout* layout = new QVBoxLayout();
        view->setLayout(layout);

        QLabel* nameLabel = new QLabel(name);
        nameLabel->setAlignment(Qt::AlignmentFlag::AlignCenter);
        layout->addWidget(nameLabel);

        QHBoxLayout* rowId = new QHBoxLayout();
        rowId->addWidget(new QLabel(xmlFieldId));
        QLabel* idLabel = new QLabel(QString::number(id));
        rowId->addWidget(idLabel);
        rowId->setStretchFactor(idLabel, 1);
        layout->addLayout(rowId);
        layout->addWidget(new QLabel(description));

        QMapIterator<QString, QLineEdit*> i(fields);
        while (i.hasNext())
        {
            i.next();
            QLabel* fieldNameLabel = new QLabel(i.key());
            QHBoxLayout* rowLayout = new QHBoxLayout();

            rowLayout->addWidget(fieldNameLabel);
            rowLayout->addWidget(i.value());
            layout->addLayout(rowLayout);
        }
    }

    return view;
}

QString MsgView::getName() const { return name; }

QMap<QString, QLineEdit*> MsgView::getFields() const { return fields; }
