#pragma once

#include <QGroupBox>
#include <QLineEdit>
#include <QRadioButton>

#include "Modules/DefaultModule/DefaultModule.h"
#include "Modules/SkywardHubStrings.h"

namespace Ui
{
class MavlinkRocketMsgTestingModule;
}

class MsgView : public QObject
{
    Q_OBJECT

public:
    MsgView(const XmlObject *obj, QString name);
    ~MsgView();
    QWidget *getView();

    QString getName() const;

    QMap<QString, QLineEdit *> getFields() const;

private:
    QString description;
    QString name;
    int id;
    QMap<QString, QLineEdit *> fields;
    QWidget *view = nullptr;

    QString xmlFieldId          = "id";
    QString xmlFieldDescr       = "description";
    QString xmlFieldElementName = "field";
    QString xmlFieldName        = "name";
};

class MavlinkRocketMsgTestingModule : public DefaultModule
{
    Q_OBJECT

public:
    explicit MavlinkRocketMsgTestingModule(QWidget *parent = nullptr);
    ~MavlinkRocketMsgTestingModule();

    QWidget *toWidget() override;

    XmlObject toXmlObject() override;
    void fromXmlObject(const XmlObject &xmlObject) override;

protected:
    void init();
    void createNewButton(const QString &txt, QLayout *container);
    void connectRadioButton(QRadioButton *radiobutton);
    void setSelectedMsg(const QString &msgName);
    void onSendMsgClicked();
    void onLoadFileClicked();
    bool testMavlinkEncodeAndDecode(const XmlObject &xmlMsg);
    //    mavlink_message_t genericPack(string funcName, char *params[]);
    QString getCurrentTopic() const;
    void createViewFromXmlMessages(QList<XmlObject *> messagesList);
    QGroupBox *getOrCreateGBox(const QString &title);
    void clearView();
    XmlObject msgToXml() const;

protected slots:
    void onRadioButtonClicked(QRadioButton *radiobutton);

private:
    Ui::MavlinkRocketMsgTestingModule *ui;

    MsgView *currentMsgView = nullptr;

    QString defaultMavlinkFileName = "mavlink.xml";
    QString xmlChildName           = "message";
    QString xmlFieldType           = "msg_type";
    QString xmlFieldName           = "name";
    XmlObject mavlinkMsgDefinitionFile;

    QMap<QString, QGroupBox *> viewGroupList;
};
