#pragma once

#include <QStringList>

class Module;

enum ModuleId
{
    EMPTY,
    SPLITTER,
    SKYWARDHUB,
    COMMANDPAD,
    COMPACT_COMMAND_PAD,
    CSV_LOGGER,
    BROKERTEST,
    GRAPH,
    OUTCOMINGMESSAGEVIEWER,
    INCOMINGMESSAGESVIEWER,
    MAVLINK,
    FILESTREAM,
    ORIENTATION_VISUALIZER,
    STATEVIEWER,
    VALUESCONVERTERVIEWER,
    MAVLINK_RCK_TESTING,
    TIMER_CONTROLLER,
    MODULETEST,
    TABS,
    FILTER_SELECTOR,
};

enum ModuleCategory
{
    HIDDEN,
    DEFAULT,
    UTILITY,
    DATAVISUAL,
    DATASOURCE,
    HOME
};

class ModuleInfo
{
public:
    ModuleInfo();
    ModuleInfo(const ModuleInfo& other);
    ModuleInfo(const ModuleId& id, const QString& name);
    ModuleInfo(const ModuleId& id, const QString& name,
               ModuleCategory category);

    ModuleId getId() const;
    void setId(const ModuleId& value);

    QString getModuleName() const;
    void setModuleName(const QString& value);

    std::function<Module*()> getFactory() const;
    void setFactory(const std::function<Module*()>& value);

    QStringList getModuleSourceFiles() const;
    void addModuleSourceFiles(const QString& value);

    void operator=(const ModuleInfo& other);

    ModuleCategory getModuleCategory() const;
    void setCategory(const ModuleCategory& value);

protected:
    void copy(const ModuleInfo& other);

private:
    ModuleId moduleId;
    ModuleCategory mCategory = ModuleCategory::DEFAULT;
    QString moduleName;
    std::function<Module*()> factory;
    QStringList moduleSourceFiles;
};
