#include "ModulesList.h"

#include <Components/FilterSelector/FilterSelector.h>
#include <Components/ModulesPicker/ModulesPicker.h>
#include <Modules/CommandPad/CommandPad.h>
#include <Modules/CompactCommandPad/CompactCommandPad.h>
#include <Modules/CsvLogger/CsvLogger.h>
#include <Modules/Empty/EmptyModule.h>
#include <Modules/Graph/Graph.h>
#include <Modules/IncomingMessagesViewer/IncomingMessagesViewerModule.h>
#include <Modules/Mavlink/MavlinkModule.h>
#include <Modules/Mavlink/MavlinkRocketMsgTestingModule.h>
#include <Modules/OrientationVisualizer/OrientationVisualizer.h>
#include <Modules/OutgoingMessagesViewer/OutgoingMessagesViewerModule.h>
#include <Modules/SkywardHub/SkywardHubModule.h>
#include <Modules/Splitter/Splitter.h>
#include <Modules/StateViewer/StateViewer.h>
#include <Modules/Tabs/TabsModule.h>
#include <Modules/Test/TestModule.h>
#include <Modules/TimerController/TimerControllerModule.h>
#include <Modules/ValuesConverterViewer/ValuesConverterViewerModule.h>

void ModulesList::createModuleList()
{
    /**
     * addModule(MODULE UNIQUE ID, MODULE UNIQUE NAME, a function that
     * instantiate your module) If you are creating a new Module, please add
     * it's id in the ModuleId enum (you can find it in moduleslist.h)
     *
     * WARNING!
     * - Each ID must be unique
     * - Each ModuleName must be unique and WITHOUT spaces
     * - A Module must NOT include a file outside its folder
     * - The ModuleSourceFiles allow you to specify which files will be included
     *   (all the files that match the strings you provide will be included)
     */

    ModuleInfo empty(ModuleId::EMPTY, "EmptyModule");
    empty.setFactory([]() { return new EmptyModule(); });
    empty.addModuleSourceFiles("Modules/Empty/");
    addModuleInfo(empty);

    ModuleInfo commandPad(ModuleId::COMMANDPAD, "CommandPad",
                          ModuleCategory::UTILITY);
    commandPad.setFactory([]() { return new CommandPad(); });
    commandPad.addModuleSourceFiles("Modules/CommandPad/");
    addModuleInfo(commandPad);

    ModuleInfo compactCommandPad(ModuleId::COMPACT_COMMAND_PAD,
                                 "CompactCommandPad",
                                 ModuleCategory::UTILITY);
    compactCommandPad.setFactory([]() { return new CompactCommandPad(); });
    compactCommandPad.addModuleSourceFiles("Modules/CompactCommandPad/");
    addModuleInfo(compactCommandPad);

    ModuleInfo csvLogger(ModuleId::CSV_LOGGER,
                                 "CsvLogger",
                                 ModuleCategory::DATASOURCE);
    csvLogger.setFactory([]() { return new CsvLogger(); });
    csvLogger.addModuleSourceFiles("Modules/CsvLogger/");
    addModuleInfo(csvLogger);

    ModuleInfo splitter(ModuleId::SPLITTER, "Splitter");
    splitter.setFactory([]() { return new Splitter(); });
    splitter.addModuleSourceFiles("Modules/Splitter/");
    addModuleInfo(splitter);

    ModuleInfo hub(ModuleId::SKYWARDHUB, "SkywardHub", ModuleCategory::HOME);
    hub.setFactory([]() { return new SkywardHubModule(); });
    hub.addModuleSourceFiles("Modules/SkywardHub/");
    addModuleInfo(hub);

    ModuleInfo testModule(ModuleId::BROKERTEST, "TestBroker",
                          ModuleCategory::DEFAULT);
    testModule.setFactory([]() { return new TestModule(); });
    testModule.addModuleSourceFiles("Modules/Test/");
    addModuleInfo(testModule);

    ModuleInfo graphModule(ModuleId::GRAPH, "Graph",
                           ModuleCategory::DATAVISUAL);
    graphModule.setFactory([]() { return new Graph(); });
    graphModule.addModuleSourceFiles("Modules/Graph/");
    addModuleInfo(graphModule);

    ModuleInfo outMsgViewer(ModuleId::OUTCOMINGMESSAGEVIEWER,
                            "OutgoingMessagesViewer",
                            ModuleCategory::DATAVISUAL);
    outMsgViewer.setFactory([]()
                            { return new OutgoingMessagesViewerModule(); });
    outMsgViewer.addModuleSourceFiles("Modules/OutgoingMessageViewer/");
    addModuleInfo(outMsgViewer);

    ModuleInfo inMsgViewer(ModuleId::INCOMINGMESSAGESVIEWER,
                           "IncomingMessagesViewer",
                           ModuleCategory::DATAVISUAL);
    inMsgViewer.setFactory([]() { return new IncomingMessagesViewerModule(); });
    inMsgViewer.addModuleSourceFiles("Modules/IncomingMessageViewer/");
    addModuleInfo(inMsgViewer);

    ModuleInfo mavlink(ModuleId::MAVLINK, "Mavlink",
                       ModuleCategory::DATASOURCE);
    mavlink.setFactory([]() { return new MavlinkModule(); });
    mavlink.addModuleSourceFiles("Modules/Mavlink/");
    addModuleInfo(mavlink);

    ModuleInfo mvkRckTesting(ModuleId::MAVLINK_RCK_TESTING,
                             "MavlinkRocketMsgTesting",
                             ModuleCategory::DEFAULT);
    mvkRckTesting.setFactory([]()
                             { return new MavlinkRocketMsgTestingModule(); });
    addModuleInfo(mvkRckTesting);

    ModuleInfo stateViewer(ModuleId::STATEVIEWER, "StateViewer",
                           ModuleCategory::DATAVISUAL);
    stateViewer.setFactory([]() { return new StateViewerModule(); });
    stateViewer.addModuleSourceFiles("Modules/StateViewer/");
    addModuleInfo(stateViewer);

    ModuleInfo valuesConverterViewer(ModuleId::VALUESCONVERTERVIEWER,
                                     "ValuesConverterViewer",
                                     ModuleCategory::UTILITY);
    valuesConverterViewer.setFactory(
        []() { return new ValuesConverterViewerModule(); });
    valuesConverterViewer.addModuleSourceFiles(
        "Modules/ValuesConverterViewer/");
    addModuleInfo(valuesConverterViewer);

    ModuleInfo orientationVisualizer(ModuleId::ORIENTATION_VISUALIZER,
                                     "OrientationVisualizer",
                                     ModuleCategory::UTILITY);
    orientationVisualizer.setFactory([]()
                                     { return new OrientationVisualizer(); });
    orientationVisualizer.addModuleSourceFiles(
        "Modules/OrientationVisualizer/");
    addModuleInfo(orientationVisualizer);

    ModuleInfo timerController(ModuleId::TIMER_CONTROLLER, "TimerController",
                               ModuleCategory::UTILITY);
    timerController.setFactory([]() { return new TimerControllerModule(); });
    timerController.addModuleSourceFiles("Modules/TimerController/");
    addModuleInfo(timerController);

    ModuleInfo tabs(ModuleId::TABS, "Tabs", ModuleCategory::UTILITY);
    tabs.setFactory([]() { return new TabsModule(); });
    tabs.addModuleSourceFiles("Modules/Tabs/");
    addModuleInfo(tabs);
}

// ____________________________________________________________________________________________

ModulesList::ModulesList() { createModuleList(); }

bool ModulesList::containsId(ModuleId id)
{
    for (int i = 0; i < modulesInfo.count(); i++)
    {
        if (modulesInfo[i].getId() == id)
            return true;
    }
    return false;
}

void ModulesList::addModuleInfo(const ModuleInfo& info)
{
    modulesInfo.append(info);
}

QString ModulesList::getModuleName(ModuleId id) const
{
    for (int i = 0; i < modulesInfo.count(); i++)
    {
        if (modulesInfo[i].getId() == id)
            return modulesInfo[i].getModuleName();
    }
    return "";
}

std::function<Module*()> ModulesList::getFactory(ModuleId id) const
{
    for (int i = 0; i < modulesInfo.count(); i++)
    {
        if (modulesInfo[i].getId() == id)
            return modulesInfo[i].getFactory();
    }
    return []() { return nullptr; };
}

std::function<Module*()> ModulesList::findFactoryByModuleName(
    const QString& name)
{
    for (int i = 0; i < modulesInfo.count(); i++)
    {
        if (modulesInfo[i].getModuleName() == name)
            return modulesInfo[i].getFactory();
    }
    return []() { return nullptr; };
}

QList<QString> ModulesList::getModulesNamesList()
{
    QList<QString> names;
    for (int i = 0; i < modulesInfo.count(); i++)
    {
        names.append(modulesInfo[i].getModuleName());
    }
    return names;
}

QList<ModuleInfo> ModulesList::getModulesInfo() { return modulesInfo; }
