#pragma once

#include <QList>
#include <QProcess>
#include <QString>

#include "Core/XmlObject.h"
#include "DeployerPathPicker.h"

class Deployer
{
public:
    Deployer();

    bool deploy(const QList<QString> &activeModules);
    QString getDeployReleaseFolder() const;

protected:
    QString writeBatFile() const;
    void copyPath(const QString &src, const QString &dst);

    QList<QString> loadFile(const QString &path);
    QString getProFileName() const;
    QString getSourceDir() const;
    QString getNewSourceDir() const;
    QString getNewProFilePath() const;
    QString getModuleListFilePath() const;

    void makeDeployDir() const;

    bool writeFile(const QList<QString> &text, const QString &path);
    void setupModulesFiles(const QList<QString> &activeModules);
    bool checkIfFilesIsActive(const QString &line) const;

    void produceNewProFile();
    QList<QString> makeNewProFile(const QList<QString> &oldProFile) const;

    void produceNewModuleListFile();
    QList<QString> makeNewModulesListFile(
        const QList<QString> &oldModulesListFile) const;

    bool isDeployCompletedSuccessfully();
    bool showResultMessage();

    bool isDeployConfirmed();

private:
    QList<QString> modulesFiles;
    QString mingw81_64_bin          = "C:/Qt/5.15.1/mingw81_64/bin";
    QString mingw810_64_bin         = "C:/Qt/Tools/mingw810_64/bin";
    QString relativeModuleFilesPath = "/Modules/moduleslist.cpp";

    QString proFilePath              = "";
    QString deployDir                = "";
    QString newCopySourceFolderName  = "source";
    QString newCopyReleaseFolderName = "release";

    QString proFileRestartCopyTag = "qnx:";
    QString proFileEndCopyTag     = "SOURCES";

    DeployerPathPicker deployerPicker;
};
