#include "DeployerPathPicker.h"

#include <QFileDialog>

#include "ui_DeployerPathPicker.h"

DeployerPathPicker::DeployerPathPicker(QWidget *parent)
    : QDialog(parent), ui(new Ui::DeployerPathPicker)
{
    ui->setupUi(this);
    // setAttribute(Qt::WA_DeleteOnClose, true); // WA_DeleteOnClose is
    // set to true, so this widget will be deleted on close

    connect(ui->pushButton_selectDeployFolder, &QPushButton::clicked, this,
            &DeployerPathPicker::onSelectDeployFolderClicked);
    connect(ui->pushButton_selectProFilePath, &QPushButton::clicked, this,
            &DeployerPathPicker::onSelectProFilePathClicked);
    connect(ui->pushButton_confirm, &QPushButton::clicked, this,
            &DeployerPathPicker::onConfirmButtonClicked);
}

DeployerPathPicker::~DeployerPathPicker() { delete ui; }

QString DeployerPathPicker::getDeploymentFolder() const
{
    return ui->textEdit_DeploymentFolder->toPlainText().trimmed();
}

QString DeployerPathPicker::getProFilePathFolder() const
{
    return ui->textEdit_proFilePath->toPlainText().trimmed();
}

void DeployerPathPicker::onSelectDeployFolderClicked()
{
    QString deployDir = QFileDialog::getExistingDirectory();
    setDeploymentFolder(deployDir);
}

void DeployerPathPicker::onSelectProFilePathClicked()
{
    QString proFilePath = QFileDialog::getOpenFileName(
        nullptr, "Select .pro file", "", "(*.pro)");
    setProFilePathFolder(proFilePath);
}

void DeployerPathPicker::setDeploymentFolder(const QString &value)
{
    ui->textEdit_DeploymentFolder->setText(value);
}

void DeployerPathPicker::setProFilePathFolder(const QString &value)
{
    ui->textEdit_proFilePath->setText(value);
}

void DeployerPathPicker::onConfirmButtonClicked()
{
    QString deployDir   = getDeploymentFolder();
    QString proFilePath = getProFilePathFolder();

    if (deployDir.isEmpty() || proFilePath.isEmpty())
    {
        setPickConfirmed(false);
    }
    setPickConfirmed(true);
    this->close();  // In order to autodelete on close, WA_DeleteOnClose must be
                    // set to true.
}

bool DeployerPathPicker::getPickConfirmed() const { return pickConfirmed; }

void DeployerPathPicker::setPickConfirmed(bool value) { pickConfirmed = value; }

bool DeployerPathPicker::pickDirectory()
{
    setPickConfirmed(false);
    this->exec();
    return getPickConfirmed();
}
