#include "TestModule.h"

#include <Core/MessageBroker/MessageBroker.h>
#include <Core/ModulesManager/ModulesManager.h>

#include "Components/ContextMenuSeparator/ContextMenuSeparator.h"
#include "ui_TestModule.h"

TestModule::TestModule(QWidget* parent)
    : DefaultModule(parent), ui(new Ui::TestModule)
{
    ui->setupUi(this);
    defaultContextMenuSetup();

    maxChar = ui->maxChar_spinBox->value();
    connect(ui->maxChar_spinBox, SIGNAL(valueChanged(int)), this,
            SLOT(onMaxCharChanged(int)));
}

TestModule::~TestModule() { delete ui; }

QWidget* TestModule::toWidget() { return this; }

XmlObject TestModule::toXmlObject()
{
    return XmlObject(getName(ModuleId::BROKERTEST));
}

void TestModule::fromXmlObject(const XmlObject& xmlObject)
{
    Q_UNUSED(xmlObject);
}

void TestModule::on_publish_button_clicked()
{
    QString topic   = ui->topic_lineEdit->text().trimmed();
    QString payload = ui->payload_lineEdit->text().trimmed();
    Message message(Topic{topic});
    auto valueField = Field(payload);
    message.setField("value", valueField);
    auto timestampField =
        Field((uint64_t)QTime::currentTime().msecsSinceStartOfDay());
    message.setField("timestamp", timestampField);
    getCore()->getMessageBroker()->publish(message);
}

void TestModule::on_subscribe_button_clicked()
{
    getCore()->getMessageBroker()->subscribe(
        Filter::fromString(ui->subscribe_lineEdit->text().trimmed()), this,
        [this](const Message& message, const Filter& filter)
        {
            QString oldText = ui->textArea->toPlainText();
            QString line =
                QDateTime::currentDateTime().toString("hh.mm.ss (zzz): ") +
                message.toString();
            QString newText(line + '\n' + oldText);
            ui->textArea->setPlainText(newText.mid(0, maxChar));
        });
}

void TestModule::on_unsubscribe_button_clicked()
{
    getCore()->getMessageBroker()->unsubscribe(
        Filter::fromString(ui->subscribe_lineEdit->text().trimmed()), this);
}

void TestModule::onMaxCharChanged(int value) { maxChar = value; }
